@@ -105,11 +105,9 @@ def test_breadcrumb_extra_scrubbing(sentry_init, capture_events):
         "password": "[Filtered]",
     }
 
-    assert event["_meta"] == {
-        "extra": {"auth": {"": {"rem": [["!config", "s"]]}}},
-        "breadcrumbs": {
-            "values": {"0": {"data": {"password": {"": {"rem": [["!config", "s"]]}}}}}
-        },
+    assert event["_meta"]["extra"]["auth"] == {"": {"rem": [["!config", "s"]]}}
+    assert event["_meta"]["breadcrumbs"] == {
+        "values": {"0": {"data": {"password": {"": {"rem": [["!config", "s"]]}}}}}
     }
 
 
@@ -124,8 +122,8 @@ def test_span_data_scrubbing(sentry_init, capture_events):
 
     (event,) = events
     assert event["spans"][0]["data"] == {"password": "[Filtered]", "datafoo": "databar"}
-    assert event["_meta"] == {
-        "spans": {"0": {"data": {"password": {"": {"rem": [["!config", "s"]]}}}}}
+    assert event["_meta"]["spans"] == {
+        "0": {"data": {"password": {"": {"rem": [["!config", "s"]]}}}}
     }
 
 
@@ -171,3 +169,18 @@ def test_scrubbing_doesnt_affect_local_vars(sentry_init, capture_events):
     (frame,) = frames
     assert frame["vars"]["password"] == "[Filtered]"
     assert password == "cat123"
+
+
+def test_recursive_event_scrubber(sentry_init, capture_events):
+    sentry_init(event_scrubber=EventScrubber(recursive=True))
+    events = capture_events()
+    complex_structure = {
+        "deep": {
+            "deeper": [{"deepest": {"password": "my_darkest_secret"}}],
+        },
+    }
+
+    capture_event({"extra": complex_structure})
+
+    (event,) = events
+    assert event["extra"]["deep"]["deeper"][0]["deepest"]["password"] == "'[Filtered]'"
